#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	//create your own enum type or use any other .net enum
	public enum KinectJoints
	{
		HipCenter,
        Spine,
        ShoulderCenter,
        Head,
        ShoulderLeft,
        ElbowLeft,
        WristLeft,
        HandLeft,
        ShoulderRight,
        ElbowRight,
        WristRight,
        HandRight,
        HipLeft,
        KneeLeft,
        AnkleLeft,
        FootLeft,
        HipRight,
        KneeRight,
        AnkleRight,
        FootRight,
	}
	//Please rename your Enum Type to avoid 
	//numerous "MyEnum"s in the system

	#region PluginInfo
	[PluginInfo(Name = "SelectJoint", Category = "3D Kinect Microsoft", Version = "Static", Help = "Select Joint by enum for Kinect Microsoft", Tags = "Select, Joint")]
	#endregion PluginInfo
	public class SelectJointNode : IPluginEvaluate
	{
		#region fields & pins

		[Input("Input")]
		ISpread<Vector3D> FXYZInput;
		
		[Input("Select", DefaultEnumEntry = "HipCenter")]
		IDiffSpread<KinectJoints> FInput;
				
		[Output("Output")]
		ISpread<Vector3D> FXYZOutput;
		
		[Output("Name")]
		ISpread<string> FNameOutput;

		[Output("Index")]
		ISpread<int> FOrdOutput;
		
		[Output("Users")]
		ISpread<int> FUsersOutput;
		
		[Output("Joints")]
		ISpread<int> FJointsOutput;
		
		[Import()]
		ILogger Flogger;
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			
			int sUsers = (FXYZInput.SliceCount/20);
			int sJoints = FInput.SliceCount;
			int sMax = (FXYZInput.SliceCount/20) * FInput.SliceCount;
			
			
			FXYZOutput.SliceCount = sMax;
			FNameOutput.SliceCount = sMax;
			FOrdOutput.SliceCount = sMax;

			if (FInput.IsChanged) 
			{
				for (int u = 0; u < sUsers; u++) 
				{
					for (int j = 0; j < sJoints; j++)
					{
						FNameOutput[u,j] = Enum.GetName(typeof(KinectJoints), FInput[u,j]);
						FOrdOutput[u,j] = (int)FInput[u,j];
						FXYZOutput[u,j] = FXYZInput[FOrdOutput[u,j]+20*u];
					}
					
			
					FUsersOutput[u] = sUsers;
					FJointsOutput[u] = sJoints;
				}
				Flogger.Log(LogType.Debug, "Input was changed");
			}
		}
	}
}
